@@ -9,7 +9,9 @@ from xmlrpc.server import SimpleXMLRPCServer
 from xmlrpc.server import SimpleXMLRPCRequestHandler
 import xmlrpc.client
 import socket
+#from socketserver import ThreadingMixIn
 import logging
+
 from pyhomematic import devicetypes
 from pyhomematic.devicetypes.generic import HMChannel
 
@@ -280,6 +282,10 @@ class RPCFunctions():
             self._devices_raw[remote] = []
         if self.systemcallback:
             self.systemcallback('listDevices', interface_id)
+
+        # return empty list for HmIP, as currently the maximum lenght is limited to 8192 bytes  (see #318 for details)
+        if self.remotes.get(remote, {}).get('port') in [2010, 32010, 42010]:
+            return []
         return self._devices_raw[remote]
 
     def newDevices(self, interface_id, dev_descriptions):
@@ -293,7 +299,11 @@ class RPCFunctions():
             self._devices_raw_dict[remote] = {}
         if remote not in self._paramsets:
             self._paramsets[remote] = {}
+        hmip = self.remotes.get(remote, {}).get('port') in [2010, 32010, 42010]
         for d in dev_descriptions:
+            if hmip:
+                if d in self._devices_raw[remote]:
+                    continue
             self._devices_raw[remote].append(d)
             self._devices_raw_dict[remote][d['ADDRESS']] = d
             self._paramsets[remote][d['ADDRESS']] = {}
@@ -348,7 +358,7 @@ class RPCFunctions():
             self.systemcallback('readdedDevice', interface_id, addresses)
         return True
 
-    def jsonRpcPost(self, host, jsonport, method, params={}):
+    def jsonRpcPost(self, host, jsonport, method, params={}, verify=False):
         LOG.debug("RPCFunctions.jsonRpcPost: Method: %s" % method)
         try:
             payload = json.dumps(
@@ -356,11 +366,19 @@ class RPCFunctions():
 
             headers = {"Content-Type": 'application/json',
                        "Content-Length": len(payload)}
-            apiendpoint = "http://%s:%s%s" % (host, jsonport, JSONRPC_URL)
+            ctx = None
+            if jsonport == 443:
+                apiendpoint = "https://%s:%s%s" % (host, jsonport, JSONRPC_URL)
+                if not verify:
+                    ctx = ssl.create_default_context()
+                    ctx.check_hostname = False
+                    ctx.verify_mode = ssl.CERT_NONE
+            else:
+                apiendpoint = "http://%s:%s%s" % (host, jsonport, JSONRPC_URL)
             LOG.debug("RPCFunctions.jsonRpcPost: API-Endpoint: %s" %
                       apiendpoint)
             req = urllib.request.Request(apiendpoint, payload, headers)
-            resp = urllib.request.urlopen(req)
+            resp = urllib.request.urlopen(req, context=ctx)
             if resp.status == 200:
                 try:
                     return json.loads(resp.read().decode('utf-8'))
@@ -440,6 +458,10 @@ class RPCFunctions():
                             if i.get('address') in self.devices[remote]:
                                 self.devices[remote][
                                     i['address']].NAME = i['name']
+                                for channel_device_response in i['channels']:
+                                    name = channel_device_response['name']
+                                    self.devices_all[remote][channel_device_response['address']].NAME = name
+
                         except Exception as err:
                             LOG.warning(
                                 "RPCFunctions.addDeviceNames: Exception: %s" % str(err))
@@ -494,7 +516,7 @@ class LockingServerProxy(xmlrpc.client.ServerProxy):
         self.lock = threading.Lock()
         if self._ssl and not self._verify_ssl and self._verify_ssl is not None:
             kwargs['context'] = ssl._create_unverified_context()
-        xmlrpc.client.ServerProxy.__init__(self, *args, **kwargs)
+        xmlrpc.client.ServerProxy.__init__(self, encoding="ISO-8859-1", *args, **kwargs)
         urlcomponents = urllib.parse.urlparse(args[0])
         self._remoteip = urlcomponents.hostname
         self._remoteport = urlcomponents.port
@@ -520,7 +542,6 @@ class LockingServerProxy(xmlrpc.client.ServerProxy):
         """
         Magic method dispatcher
         """
-
         return xmlrpc.client._Method(self.__request, *args, **kwargs)
 
 # Restrict to particular paths.
@@ -561,8 +582,44 @@ class ServerThread(threading.Thread):
         self.proxies = {}
         self.failed_inits = []
 
-        # Create proxies to interact with CCU / Homegear
-        LOG.debug("__init__: Creating proxies")
+        self.createProxies()
+        if not self.proxies:
+            LOG.warning("No proxies available. Aborting.")
+            raise Exception
+
+        self._rpcfunctions = RPCFunctions(devicefile=self._devicefile,
+                                          paramsetfile=self._paramsetfile,
+                                          proxies=self.proxies,
+                                          remotes=self.remotes,
+                                          eventcallback=self.eventcallback,
+                                          systemcallback=self.systemcallback,
+                                          resolveparamsets=self.resolveparamsets)
+
+        # Setup server to handle requests from CCU / Homegear
+        LOG.debug("ServerThread.__init__: Setting up server")
+        # class SimpleThreadedXMLRPCServer(ThreadingMixIn, SimpleXMLRPCServer):
+        #     pass
+        # self.server = SimpleThreadedXMLRPCServer((self._local, self._localport),
+        #                                          requestHandler=RequestHandler,
+        #                                          logRequests=False)
+        self.server = SimpleXMLRPCServer((self._local, self._localport),
+                                         requestHandler=RequestHandler,
+                                         logRequests=False)
+        self._localport = self.server.socket.getsockname()[1]
+        self.server.register_introspection_functions()
+        self.server.register_multicall_functions()
+        LOG.debug("ServerThread.__init__: Registering RPC functions")
+        self.server.register_instance(
+            self._rpcfunctions, allow_dotted_names=True)
+
+    def run(self):
+        LOG.info("Starting server at http://%s:%i" %
+                 (self._local, self._localport))
+        self.server.serve_forever()
+
+    def createProxies(self):
+        """Create proxies to interact with CCU / Homegear"""
+        LOG.debug("createProxies: Creating proxies")
         for remote, host in self.remotes.items():
             # Initialize XML-RPC
             try:
@@ -594,6 +651,7 @@ class ServerThread(threading.Thread):
                 LOG.warning("Failed connecting to proxy at http://%s:%i%s" %
                             (host['ip'], host['port'], host['path']))
                 LOG.debug("__init__: Exception: %s" % str(err))
+                # pylint: disable=raise-missing-from
                 raise Exception
             try:
                 host['type'] = BACKEND_UNKNOWN
@@ -607,34 +665,10 @@ class ServerThread(threading.Thread):
                 LOG.warning("__init__: Failed to detect backend type: %s" % str(err))
                 host['type'] = BACKEND_UNKNOWN
 
-        if not self.proxies:
-            LOG.warning("No proxies available. Aborting.")
-            raise Exception
-
-        self._rpcfunctions = RPCFunctions(devicefile=self._devicefile,
-                                          paramsetfile=self._paramsetfile,
-                                          proxies=self.proxies,
-                                          remotes=self.remotes,
-                                          eventcallback=self.eventcallback,
-                                          systemcallback=self.systemcallback,
-                                          resolveparamsets=self.resolveparamsets)
-
-        # Setup server to handle requests from CCU / Homegear
-        LOG.debug("ServerThread.__init__: Setting up server")
-        self.server = SimpleXMLRPCServer((self._local, self._localport),
-                                         requestHandler=RequestHandler,
-                                         logRequests=False)
-        self._localport = self.server.socket.getsockname()[1]
-        self.server.register_introspection_functions()
-        self.server.register_multicall_functions()
-        LOG.debug("ServerThread.__init__: Registering RPC functions")
-        self.server.register_instance(
-            self._rpcfunctions, allow_dotted_names=True)
-
-    def run(self):
-        LOG.info("Starting server at http://%s:%i" %
-                 (self._local, self._localport))
-        self.server.serve_forever()
+    def clearProxies(self):
+        """Remove existing proxy objects."""
+        LOG.debug("clearProxies: Clearing proxies")
+        self.proxies.clear()
 
     def proxyInit(self):
         """
@@ -644,6 +678,7 @@ class ServerThread(threading.Thread):
         # the receiver) to receive events. XML RPC server has to be running.
         for interface_id, proxy in self.proxies.items():
             if proxy._skipinit:
+                LOG.warning("Skipping init for %s", interface_id)
                 continue
             if proxy._callbackip and proxy._callbackport:
                 callbackip = proxy._callbackip
@@ -654,20 +689,25 @@ class ServerThread(threading.Thread):
             LOG.debug("ServerThread.proxyInit: init('http://%s:%i', '%s')" %
                       (callbackip, callbackport, interface_id))
             try:
+                # For HomeMatic IP, init is not working correctly. We fetch the device list and create
+                # the device objects before the init is performed.
+                if proxy._remoteport in [2010, 32010, 42010]:
+                    dev_list = proxy.listDevices(interface_id)
+                    self._rpcfunctions.newDevices(interface_id=interface_id, dev_descriptions=dev_list)
                 proxy.init("http://%s:%i" %
                            (callbackip, callbackport), interface_id)
-                LOG.info("Proxy initialized")
+                LOG.info("Proxy for %s initialized", interface_id)
             except Exception as err:
                 LOG.debug("proxyInit: Exception: %s" % str(err))
-                LOG.warning("Failed to initialize proxy")
+                LOG.warning("Failed to initialize proxy for %s", interface_id)
                 self.failed_inits.append(interface_id)
 
-    def stop(self):
-        """To stop the server we de-init from the CCU / Homegear, then shut down our XML-RPC server."""
+    def proxyDeInit(self):
+        """De-Init from the proxies."""
         stopped = []
         for interface_id, proxy in self.proxies.items():
             if interface_id in self.failed_inits:
-                LOG.warning("ServerThread.stop: Not performing de-init for %s" % interface_id)
+                LOG.warning("ServerThread.proxyDeInit: Not performing de-init for %s", interface_id)
                 continue
             if proxy._callbackip and proxy._callbackport:
                 callbackip = proxy._callbackip
@@ -676,21 +716,25 @@ class ServerThread(threading.Thread):
                 callbackip = proxy._localip
                 callbackport = self._localport
             remote = "http://%s:%i" % (callbackip, callbackport)
-            LOG.debug("ServerThread.stop: init('%s')" % remote)
-            if not callbackip in stopped:
+            LOG.debug("ServerThread.proxyDeInit: init('%s')", remote)
+            if not interface_id in stopped:
                 try:
                     proxy.init(remote)
-                    stopped.append(callbackip)
-                    LOG.info("Proxy de-initialized: %s" % remote)
+                    stopped.append(interface_id)
+                    LOG.info("proxyDeInit: Proxy for %s de-initialized: %s", interface_id, remote)
                 except Exception as err:
-                    LOG.debug("proxyInit: Exception: %s" % str(err))
-                    LOG.warning("Failed to de-initialize proxy")
-        self.proxies.clear()
+                    LOG.debug("proxyDeInit: Exception: %s", err)
+                    LOG.warning("proxyDeInit: Failed to de-initialize proxy")
+
+    def stop(self):
+        """To stop the server we de-init from the CCU / Homegear, then shut down our XML-RPC server."""
+        self.proxyDeInit()
+        self.clearProxies()
         LOG.info("Shutting down server")
         self.server.shutdown()
         LOG.debug("ServerThread.stop: Stopping ServerThread")
         self.server.server_close()
-        LOG.info("Server stopped")
+        LOG.info("HomeMatic XML-RPC Server stopped")
 
     def parseCCUSysVar(self, data):
         """Helper to parse type of system variables of CCU"""
@@ -963,9 +1007,13 @@ class ServerThread(threading.Thread):
                 "ServerThread.homegearCheckInit: Exception: %s" % str(err))
             return False
 
-    def putParamset(self, remote, address, paramset, value):
+    def putParamset(self, remote, address, paramset, value, rx_mode=None):
         """Set paramsets manually"""
         try:
-            return self.proxies["%s-%s" % (self._interface_id, remote)].putParamset(address, paramset, value)
+            proxy = self.proxies["%s-%s" % (self._interface_id, remote)]
+            if rx_mode is None:
+                return proxy.putParamset(address, paramset, value)
+            else:
+                return proxy.putParamset(address, paramset, value, rx_mode)
         except Exception as err:
             LOG.debug("ServerThread.putParamset: Exception: %s" % str(err))
