@@ -1,10 +1,12 @@
+
 import logging
 from pyhomematic.devicetypes.generic import HMDevice
 from pyhomematic.devicetypes.sensors import HMSensor
 from pyhomematic.devicetypes.misc import HMEvent
 from pyhomematic.devicetypes.helper import (
     HelperWorking, HelperActorState, HelperActorLevel, HelperActorBlindTilt, HelperActionOnTime,
-    HelperActionPress, HelperEventRemote, HelperWired, HelperRssiPeer, HelperRssiDevice)
+    HelperActionPress, HelperEventRemote, HelperWired, HelperRssiPeer, HelperRssiDevice, HelperDeviceTemperature,
+    HelperInhibit, HelperLowBatIP)
 
 LOG = logging.getLogger(__name__)
 
@@ -33,7 +35,7 @@ class GenericBlind(HMActor, HelperActorLevel):
         self.actionNodeData("STOP", True, channel)
 
 
-class Blind(GenericBlind, HelperWorking, HelperRssiPeer):
+class Blind(GenericBlind, HelperInhibit, HelperWorking, HelperRssiPeer):
     """
     Blind switch that raises and lowers roller shutters or window blinds.
     """
@@ -56,6 +58,7 @@ class IPBlind(GenericBlind, HelperRssiPeer):
                                    "LEVEL_STATUS": self.ELEMENT,
                                    "SECTION": self.ELEMENT})
         self.ACTIONNODE.update({"STOP": self.ELEMENT})
+        self.SENSORNODE.update({"LEVEL": [3]})
         self.WRITENODE.update({"LEVEL": self.ELEMENT})
 
 
@@ -103,6 +106,14 @@ class IPKeyBlindMulti(KeyBlind):
 
 
 class IPKeyBlindTilt(IPKeyBlind, HelperActorBlindTilt):
+    """
+    Blind switch that raises, lowers and adjusts the slat position of shutters or blinds.
+    """
+    def __init__(self, device_description, proxy, resolveparamsets=False):
+        super().__init__(device_description, proxy, resolveparamsets)
+
+        # init metadata
+        self.SENSORNODE.update({"LEVEL_2": [3]})
 
     def close_slats(self, channel=None):
         """Move the shutter up all the way."""
@@ -127,18 +138,18 @@ class GenericDimmer(HMActor, HelperActorLevel):
         self.set_level(0.0, channel)
 
 
-class Dimmer(GenericDimmer, HelperWorking):
+class Dimmer(GenericDimmer, HelperInhibit, HelperWorking):
     """
     Dimmer switch that controls level of light brightness.
     """
     @property
     def ELEMENT(self):
-        if "Dim2L" in self._TYPE or "Dim2T" in self._TYPE  or self._TYPE == "HM-DW-WM" or self._TYPE == "HM-LC-DW-WM":
+        if "Dim2L" in self._TYPE or "Dim2T" in self._TYPE  or self._TYPE == "HM-DW-WM":
             return [1, 2]
         return [1]
 
 
-class KeyDimmer(GenericDimmer, HelperWorking, HelperActionPress):
+class KeyDimmer(GenericDimmer, HelperInhibit, HelperWorking, HelperActionPress):
     """
     Dimmer switch that controls level of light brightness.
     """
@@ -165,7 +176,7 @@ class IPDimmer(GenericDimmer):
         return [2]
 
 
-class IPKeyDimmer(GenericDimmer, HelperWorking, HelperActionPress):
+class IPKeyDimmer(GenericDimmer, HelperActionPress):
     """
     IP Dimmer with buttons switch that controls level of light brightness.
     """
@@ -174,7 +185,7 @@ class IPKeyDimmer(GenericDimmer, HelperWorking, HelperActionPress):
 
         # init metadata
         self.EVENTNODE.update({"PRESS_SHORT": [1, 2],
-                               "PRESS_LONG_RELEASE": [1, 2]})
+                               "PRESS_LONG": [1, 2]})
 
     @property
     def ELEMENT(self):
@@ -203,7 +214,7 @@ class GenericSwitch(HMActor, HelperActorState):
         self.set_state(False, channel)
 
 
-class Rain(GenericSwitch, HelperWorking):
+class Rain(GenericSwitch, HelperInhibit, HelperWorking):
     """Rain / Heat sensor with heating switch"""
     def __init__(self, device_description, proxy, resolveparamsets=False):
         super().__init__(device_description, proxy, resolveparamsets)
@@ -219,7 +230,7 @@ class Rain(GenericSwitch, HelperWorking):
         return [2]
 
 
-class Switch(GenericSwitch, HelperWorking, HelperRssiPeer):
+class Switch(GenericSwitch, HelperInhibit, HelperWorking, HelperRssiPeer):
     """
     Switch turning plugged in device on or off.
     """
@@ -239,12 +250,19 @@ class Switch(GenericSwitch, HelperWorking, HelperRssiPeer):
             return [13, 14, 15, 16, 17, 18, 19]
         return [1]
 
-
-class IOSwitch(GenericSwitch, HelperWorking, HelperEventRemote, HelperWired):
+class IOSwitchWireless(GenericSwitch, HelperInhibit, HelperWorking, HelperEventRemote, HelperRssiPeer):
     """
-    Switch turning attached device on or off.
+    Switch turning attached device on or off. Can controll relais and buttons independently.
     """
+    @property
+    def ELEMENT(self):
+        return [1, 2]
 
+
+class IOSwitchNoInhibit(GenericSwitch, HelperWorking, HelperEventRemote, HelperWired):
+    """
+    Switch turning attached device on or off and not having a inhibit function.
+    """
     def __init__(self, device_description, proxy, resolveparamsets=False):
         self._doc = []
         super().__init__(device_description, proxy, resolveparamsets)
@@ -255,10 +273,20 @@ class IOSwitch(GenericSwitch, HelperWorking, HelperEventRemote, HelperWired):
 
     @property
     def ELEMENT(self):
-        if "HMW-IO-12-Sw7-DR" in self.TYPE:
-            return [13, 14, 15, 16, 17, 18, 19]
         if "HMW-IO-12-FM" in self.TYPE:
             return self._doc
+        return [1]
+
+
+class IOSwitch(GenericSwitch, HelperInhibit, HelperWorking, HelperEventRemote, HelperWired):
+    """
+    Switch turning attached device on or off.
+    """
+
+    @property
+    def ELEMENT(self):
+        if "HMW-IO-12-Sw7-DR" in self.TYPE:
+            return [13, 14, 15, 16, 17, 18, 19]
         if "HMW-LC-Sw2-DR" in self.TYPE:
             return [3, 4]
         return [1]
@@ -323,7 +351,123 @@ class HMWIOSwitch(GenericSwitch, HelperWired):
         return self._doc
 
 
-class RFSiren(GenericSwitch, HelperWorking, HelperRssiPeer):
+class IPWSwitch(GenericSwitch, HelperDeviceTemperature, HelperWired):
+    """
+    IP-Wired Switch units turning attached device on or off.
+    """
+    @property
+    def ELEMENT(self):
+        if "HmIPW-DRS4" in self.TYPE:
+            # Address correct switching channels for each relais
+            return [2, 6, 10, 14]
+        elif "HmIPW-DRS8" in self.TYPE:
+            # Address correct switching channels for each relais
+            return [2, 6, 10, 14, 18, 22, 26, 30]
+        return [1]
+
+
+class IPWDimmer(GenericDimmer, HelperDeviceTemperature, HelperWired):
+    """
+    IP-Wired Dimmer switch that controls level of light brightness.
+    """
+    @property
+    def ELEMENT(self):
+        if "HmIPW-DRD3" in self._TYPE:
+            # Address correct switching channels for each relais
+            return [2, 6, 10]
+        return [1]
+
+class IPWKeyBlindMulti(KeyBlind, HelperDeviceTemperature, HelperWired):
+    """
+    Multi-blind actor HmIPW-DRBL4
+    """
+    def __init__(self, device_description, proxy, resolveparamsets=False):
+        super().__init__(device_description, proxy, resolveparamsets)
+
+        # init metadata
+        self.ATTRIBUTENODE.update({"ACTIVITY_STATE": self.ELEMENT,
+                                   "LEVEL_STATUS": self.ELEMENT,
+                                   "SECTION": self.ELEMENT})
+        self.ACTIONNODE.update({"STOP": self.ELEMENT})
+        self.WRITENODE.update({"LEVEL": self.ELEMENT})
+
+    @property
+    def ELEMENT(self):
+        return [2, 6, 10, 14]
+
+class IPWInputDevice(HMEvent, HelperDeviceTemperature, HelperWired):
+    """
+    IP-Wired component to support long / short press events and state report (e.g. if window contact or on/off switch)
+    """
+    def __init__(self, device_description, proxy, resolveparamsets=False):
+        super().__init__(device_description, proxy, resolveparamsets)
+        self._hmipw_keypress_event_channels = []
+        self._hmipw_binarysensor_channels = []
+
+        for chan in self.ELEMENT:
+            address_channel = "%s:%i" % (self._ADDRESS, chan)
+            try:
+                channel_paramset = self._proxy.getParamset(address_channel, "MASTER", 0)
+                channel_operation_mode = channel_paramset.get("CHANNEL_OPERATION_MODE") if "CHANNEL_OPERATION_MODE" in channel_paramset else 1
+
+                if channel_operation_mode == 1:
+                    self._hmipw_keypress_event_channels.append(chan)
+                elif channel_operation_mode in [2, 3]:
+                    self._hmipw_binarysensor_channels.append(chan)
+            except Exception as err:
+                LOG.error("IPWInputDevice: Failure to determine input channel operations mode of HmIPW input device %s: %s", address_channel, err)
+
+        self.ACTIONNODE.update({"PRESS_SHORT": self._hmipw_keypress_event_channels,
+                                "PRESS_LONG": self._hmipw_keypress_event_channels})
+        self.BINARYNODE.update({"STATE": self._hmipw_binarysensor_channels})
+
+    @property
+    def ELEMENT(self):
+        """ General channel definition """
+        if "HmIPW-DRI16" in self.TYPE:
+            return [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]
+        elif "HmIPW-DRI32" in self.TYPE:
+            return [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32]
+        return [1]
+
+
+class IPWIODevice(HMEvent, GenericSwitch, HelperWired):
+    """
+    IP-Wired I/O component to support long / short press events and state report (e.g. if window contact or on/off switch)
+    """
+    def __init__(self, device_description, proxy, resolveparamsets=False):
+        super().__init__(device_description, proxy, resolveparamsets)
+        self._hmipw_keypress_event_channels = []
+        self._hmipw_binarysensor_channels = []
+        self._ic = [1]
+
+        # Set Input Channels depending on Device
+        if "HmIPW-FIO6" in self.TYPE:
+            self._ic = [1, 2, 3, 4, 5, 6]
+
+        # Get Operation Mode for Input Channels
+        for chan in self._ic:
+            try:
+                if self._proxy.getParamset("%s:%i" % (self._ADDRESS, chan), "MASTER").get("CHANNEL_OPERATION_MODE", None) == 1:
+                    self._hmipw_keypress_event_channels.append(chan)
+                else:
+                    self._hmipw_binarysensor_channels.append(chan)
+            except Exception as err:
+                LOG.error("IPWIODevice: Failure to determine input channel operations mode of HmIPW input device %s:%i %s", self._ADDRESS, chan, err)
+
+        self.ACTIONNODE.update({"PRESS_SHORT": self._hmipw_keypress_event_channels,
+                                "PRESS_LONG": self._hmipw_keypress_event_channels})
+        self.BINARYNODE.update({"STATE": self._hmipw_binarysensor_channels})
+
+    @property
+    def ELEMENT(self):
+        """ General output channel definition """
+        if "HmIPW-FIO6" in self.TYPE:
+            return [8, 12, 16, 20, 24, 28]
+        return [1]
+
+
+class RFSiren(GenericSwitch, HelperInhibit, HelperWorking, HelperRssiPeer):
     """
     HM-Sec-Sir-WM Siren
     """
@@ -341,7 +485,7 @@ class RFSiren(GenericSwitch, HelperWorking, HelperRssiPeer):
         return [1, 2, 3]
 
 
-class KeyMatic(HMActor, HelperActorState, HelperRssiPeer):
+class KeyMatic(HMActor, HelperInhibit, HelperActorState, HelperRssiPeer):
     """
     Lock, Unlock or Open KeyMatic.
     """
@@ -351,7 +495,8 @@ class KeyMatic(HMActor, HelperActorState, HelperRssiPeer):
         # init metadata
         self.ACTIONNODE.update({"OPEN": self.ELEMENT})
         self.ATTRIBUTENODE.update({"STATE_UNCERTAIN": self.ELEMENT,
-                                   "ERROR": self.ELEMENT})
+                                   "ERROR": self.ELEMENT,
+                                   "LOWBAT": self.ELEMENT})
 
     def is_unlocked(self, channel=None):
         """ Returns True if KeyMatic is unlocked. """
@@ -386,11 +531,27 @@ class IPSwitch(GenericSwitch, HelperActionOnTime):
     """
     Switch turning attached device on or off.
     """
+    def __init__(self, device_description, proxy, resolveparamsets=False):
+        super().__init__(device_description, proxy, resolveparamsets)
+
+        channels = []
+        if "HMIP-PS" in self.TYPE.upper() or "HmIP-PCBS" in self.TYPE or "HmIP-DRSI1" in self.TYPE or "HmIP-FSI16" in self.TYPE:
+            channels = [1]
+        elif "HmIP-MOD-OC8" in self.TYPE:
+            channels = [1,2,3,4,5,6,7,8]
+        elif "HmIP-DRSI4" in self.TYPE:
+            channels = [1,2,3,4]
+
+        if channels:
+            self.EVENTNODE.update({"PRESS_SHORT": channels,
+                                   "PRESS_LONG": channels})
 
     @property
     def ELEMENT(self):
         if "HmIP-BSM" in self.TYPE:
             return [4]
+        elif "HmIP-PCBS2" in self.TYPE:
+            return [4, 8]
         elif "HmIP-FSM" in self.TYPE or "HmIP-FSM16" in self.TYPE:
             return [2]
         elif "HmIP-MOD-OC8" in self.TYPE:
@@ -401,6 +562,15 @@ class IPSwitch(GenericSwitch, HelperActionOnTime):
             return [3]
 
 
+class IPSwitchBattery(GenericSwitch, HelperActionOnTime, HelperLowBatIP):
+    """
+    Battery powered switch turning attached device on or off.
+    """
+    @property
+    def ELEMENT(self):
+        return [3]
+
+
 class IPKeySwitch(IPSwitch, HMEvent, HelperActionPress):
     """
     Switch turning plugged in device on or off and measuring energy consumption.
@@ -584,7 +754,7 @@ class IPKeySwitchPowermeter(IPSwitchPowermeter, HMEvent, HelperActionPress):
                                "PRESS_LONG": [1, 2]})
 
 
-class IPGarage(GenericSwitch, HMSensor):
+class IPGarage(GenericSwitch, GenericBlind, HMSensor):
     """
     HmIP-MOD-HO and HmIP-MOD-TM Garage actor
     """
@@ -594,16 +764,33 @@ class IPGarage(GenericSwitch, HMSensor):
         # init metadata
         self.SENSORNODE.update({"DOOR_STATE": [1]})
 
-    def move_up(self):
+    def is_closed(self, state):
+        """Returns whether the door is closed"""
+        # States:
+        # 0: closed
+        # 1: open
+        # 2: ventilation
+        # 3: unknown
+        if state in [2, 3]:
+            return None
+        return state == 0
+
+    def move_up(self, channel=1):
         """Opens the garage"""
+        # channel needs to be hardcoded to "1"; home assistant somehow calls the cover entity with channel=2
+        # and then the command does not work.
         return self.setValue("DOOR_COMMAND", 1, channel=1)
 
-    def stop(self):
+    def stop(self, channel=1):
         """Stop motion"""
+        # channel needs to be hardcoded to "1"; home assistant somehow calls the cover entity with channel=2
+        # and then the command does not work.
         return self.setValue("DOOR_COMMAND", 2, channel=1)
 
-    def move_down(self):
+    def move_down(self, channel=1):
         """Close the garage"""
+        # channel needs to be hardcoded to "1"; home assistant somehow calls the cover entity with channel=2
+        # and then the command does not work.
         return self.setValue("DOOR_COMMAND", 3, channel=1)
 
     def vent(self):
@@ -633,7 +820,7 @@ class IPMultiIO(IPSwitch):
 
     @property
     def ELEMENT(self):
-        return [3, 7]
+        return [2, 3, 4, 6, 7, 8]
 
 
 class ColorEffectLight(Dimmer):
@@ -869,20 +1056,30 @@ DEVICETYPES = {
     "HM-ES-PMSwX": SwitchPowermeter,
     "HMW-IO-12-Sw7-DR": IOSwitch,
     "HMW-IO-12-Sw14-DR": HMWIOSwitch,
-    "HMW-IO-12-FM": IOSwitch,
+    "HMW-IO-12-FM": IOSwitchNoInhibit,
     "HMW-LC-Sw2-DR": IOSwitch,
+    "HB-LC-Sw2PBU-FM": IOSwitchWireless,
     "HMW-LC-Bl1-DR": KeyBlind,
     "HMW-LC-Bl1-DR-2": KeyBlind,
     "HMW-LC-Dim1L-DR": KeyDimmer,
+    "HmIPW-DRS4": IPWSwitch,
+    "HmIPW-DRS8": IPWSwitch,
+    "HmIPW-DRI32": IPWInputDevice,
+    "HmIPW-DRI16": IPWInputDevice,
+    "HmIPW-FIO6": IPWIODevice,
+    "HmIPW-DRD3": IPWDimmer,
+    "HmIPW-DRBL4": IPWKeyBlindMulti,
     "HMIP-PS": IPSwitch,
     "HmIP-PS": IPSwitch,
     "HmIP-PS-CH": IPSwitch,
     "HmIP-PS-PE": IPSwitch,
     "HmIP-PS-UK": IPSwitch,
     "HmIP-PCBS": IPSwitch,
-    "HmIP-PCBS-BAT": IPSwitch,
+    "HmIP-PCBS2": IPSwitch,
+    "HmIP-PCBS-BAT": IPSwitchBattery,
     "HmIP-PMFS": IPSwitch,
     "HmIP-MOD-OC8": IPSwitch,
+    "HmIP-DRSI1": IPSwitch,
     "HmIP-DRSI4": IPSwitch,
     "HmIP-BSL": IPKeySwitchLevel,
     "HMIP-PSM": IPSwitchPowermeter,
@@ -891,6 +1088,7 @@ DEVICETYPES = {
     "HmIP-PSM-IT": IPSwitchPowermeter,
     "HmIP-PSM-PE": IPSwitchPowermeter,
     "HmIP-PSM-UK": IPSwitchPowermeter,
+    "HmIP-FSI16": IPSwitch,
     "HmIP-FSM": IPSwitchPowermeter,
     "HmIP-FSM16": IPSwitchPowermeter,
     "HmIP-BSM": IPKeySwitchPowermeter,
@@ -912,4 +1110,5 @@ DEVICETYPES = {
     "HmIP-MIOB": IPMultiIO,
     "HM-DW-WM": Dimmer,
     "HM-LC-DW-WM": ColdWarmDimmer,
+    "HB-UNI-RGB-LED-CTRL": ColorEffectLight,
 }
