@@ -52,6 +52,11 @@ class SensorHmIP(HMSensor, HelperRssiDevice, HelperLowBatIP, HelperOperatingVolt
          - low battery status (HelperLowBatIP)
          - voltage of the batteries (HelperOperatingVoltageIP)"""
 
+
+class SensorHmIPW(HMSensor, HelperWired):
+    """Homematic IP Wired sensors"""
+
+
 class SensorHmIPNoVoltage(HMSensor, HelperRssiDevice, HelperLowBatIP):
     """Some Homematic IP sensors have
          - strength of the signal received by the CCU (HelperRssiDevice).
@@ -63,6 +68,19 @@ class SensorHmIPNoVoltage(HMSensor, HelperRssiDevice, HelperLowBatIP):
          - low battery status (HelperLowBatIP)
          - but no voltage of batteries"""
 
+
+class SensorHmIPNoLowbat(HMSensor, HelperRssiDevice, HelperOperatingVoltageIP):
+    """Homematic IP sensors always have
+         - strength of the signal received by the CCU (HelperRssiDevice).
+           Be aware that HMIP devices have a reversed understanding of PEER
+           and DEVICE compared to standard HM devices.
+         - strength of the signal received by the device (HelperRssiPeer).
+           Be aware that standard HMIP devices have a reversed understanding of PEER
+           and DEVICE compared to standard HM devices.
+         - but no low battery status (HelperLowBatIP)
+         - voltage of the batteries (HelperOperatingVoltageIP)"""
+
+
 class SensorHmIPNoBattery(HMSensor, HelperRssiDevice):
     """Some Homematic IP sensors have
          - strength of the signal received by the CCU (HelperRssiDevice).
@@ -71,7 +89,7 @@ class SensorHmIPNoBattery(HMSensor, HelperRssiDevice):
          - strength of the signal received by the device (HelperRssiPeer).
            Be aware that standard HMIP devices have a reversed understanding of PEER
            and DEVICE compared to standard HM devices.
-         - low battery status (HelperLowBatIP)
+         - but no low battery status (HelperLowBatIP)
          - but no voltage of batteries"""
 
 
@@ -408,7 +426,7 @@ class MotionIP(SensorHmIP):
         return [0, 1]
 
 
-class MotionIPV2(SensorHmIP, HelperSabotageIP):
+class MotionIPV2(SensorHmIP):
     """Motion detection indoor 55 (rf ip)
        This is a binary sensor."""
 
@@ -461,6 +479,35 @@ class PresenceIP(SensorHmIP, HelperSabotageIP):
         return [0, 1]
 
 
+class PresenceIPW(SensorHmIPW):
+    """Presence detection with HmIPW-SPI.
+       This is a binary sensor."""
+
+    def __init__(self, device_description, proxy, resolveparamsets=False):
+        super().__init__(device_description, proxy, resolveparamsets)
+
+        # init metadata
+        self.BINARYNODE.update({"PRESENCE_DETECTION_STATE": self.ELEMENT,
+                                "PRESENCE_DETECTION_ACTIVE": self.ELEMENT,
+                                "CURRENT_ILLUMINATION_STATUS": self.ELEMENT,
+                                "ILLUMINATION_STATUS": self.ELEMENT})
+        self.SENSORNODE.update({"ILLUMINATION": self.ELEMENT,
+                                "CURRENT_ILLUMINATION": self.ELEMENT})
+        self.ATTRIBUTENODE.update({"ERROR_CODE": [0]})
+
+    def is_motion(self, channel=None):
+        """ Return True if motion is detected """
+        return bool(self.getBinaryData("PRESENCE_DETECTION_STATE", channel))
+
+    def get_brightness(self, channel=None):
+        """ Return brightness from 0 (dark) to 163830 (bright) """
+        return float(self.getSensorData("ILLUMINATION", channel))
+
+    @property
+    def ELEMENT(self):
+        return [1]
+
+
 class TiltIP(SensorHmIP):
     """Tilt detection with HmIP-SAM.
        This is a binary sensor."""
@@ -505,6 +552,14 @@ class RemoteMotion(SensorHm, Remote):
         return [1, 2]
 
 
+class IPRemoteMotionV2(Remote, MotionIPV2):
+    """Motion detection with buttons (hm ip).
+       This is a binary sensor."""
+
+    # pylint: disable=useless-super-delegation
+    def __init__(self, device_description, proxy, resolveparamsets=False):
+        super().__init__(device_description, proxy, resolveparamsets)
+
 class LuxSensor(SensorHm):
     """Sensor for messure LUX."""
 
@@ -596,6 +651,24 @@ class TemperatureSensor(SensorHm):
         return float(self.getSensorData("TEMPERATURE", channel))
 
 
+class HBUNISenWEA(TemperatureSensor):
+    """HB-UNI-Sen-WEA"""
+    def __init__(self, device_description, proxy, resolveparamsets=False):
+        super().__init__(device_description, proxy, resolveparamsets)
+
+        self.SENSORNODE.update({"AIR_PRESSURE": self.ELEMENT,
+                                "HUMIDITY": self.ELEMENT,
+                                "LUX": self.ELEMENT,
+                                "RAIN_COUNTER": self.ELEMENT,
+                                "WIND_SPEED": self.ELEMENT,
+                                "WIND_DIRECTION": self.ELEMENT,
+                                "WIND_DIRECTION_RANGE": self.ELEMENT,
+                                "GUST_SPEED": self.ELEMENT,
+                                "UVINDEX": self.ELEMENT,
+                                "LIGHTNING_COUNTER": self.ELEMENT,
+                                "LIGHNTING_DISTANCE": self.ELEMENT})
+
+
 class TemperatureDiffSensor(SensorHm):
     """Temperature difference Sensor."""
 
@@ -734,6 +807,26 @@ class IPWeatherSensorBasic(SensorHmIP):
         return bool(self.getAttributeData("TEMPERATURE_OUT_OF_RANGE", channel))
 
 
+class IPRainSensor(SensorHmIPNoLowbat):
+    """HomeMatic IP Rain sensor HmIP-SRD."""
+
+    def __init__(self, device_description, proxy, resolveparamsets=False):
+        super().__init__(device_description, proxy, resolveparamsets)
+
+        # init metadata
+        self.SENSORNODE.update({"ACTUAL_TEMPERATURE": [1]})
+        self.BINARYNODE.update({"RAINING": [1],
+                                "HEATER_STATE": [1]})
+        self.ATTRIBUTENODE.update({"ERROR_CODE": [0],
+                                   "OPERATING_VOLTAGE": [0]})
+
+    def get_temperature(self, channel=None):
+        return float(self.getSensorData("ACTUAL_TEMPERATURE", channel))
+
+    def is_raining(self, channel=None):
+        return bool(self.getBinaryData("RAINING", channel))
+
+
 class IPPassageSensor(SensorHmIP, HelperRssiPeer, HelperEventRemote):
     """HomeMatic IP Passage Sensor. #2 = right to left, #3 = left to right
        This is a binary sensor."""
@@ -921,8 +1014,71 @@ class IPContact(SensorHmIP, HelperBinaryState, HelperEventRemote):
             return [1, 2, 3, 4, 5, 6]
         elif "FCI1" in self._TYPE:
             return [1]
+        return [1]
 
+class IPAlarmSensor(SensorHmIP, HelperSabotageIP):
+    """Alarm Sirene that emits its Acoustic and Optical Alarm Status"""
 
+    def __init__(self, device_description, proxy, resolveparamsets=False):
+        super().__init__(device_description, proxy, resolveparamsets)
+
+        # init metadata
+        self.BINARYNODE.update({
+            "OPTICAL_ALARM_ACTIVE": [3],
+            "ACOUSTIC_ALARM_ACTIVE": [3]
+        })
+
+    def is_optical_alarm_active(self, channel=None):
+        return bool(self.getBinaryData("OPTICAL_ALARM_ACTIVE", channel))
+
+    def is_acoustic_alarm_active(self, channel=None):
+        return bool(self.getBinaryData("ACOUSTIC_ALARM_ACTIVE", channel))
+
+class ValveBox(SensorHmIP):
+    """Valve Box HmIP-FALMOT-C12"""
+
+    def __init__(self, device_description, proxy, resolveparamsets=False):
+        super().__init__(device_description, proxy, resolveparamsets)
+
+        self.SENSORNODE.update({"LEVEL": self.ELEMENT})
+
+    def get_level(self, channel=None):
+        """Return valve state from 0% to 99%"""
+        return float(self.getSensorData("LEVEL", channel))
+
+    @property
+    def ELEMENT(self):
+        return [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
+
+class IPLanRouter(HMSensor):
+    """ HmIP Lan Router HmIP-HAP"""
+
+    def __init__(self, device_description, proxy, resolveparamsets=False):
+        super().__init__(device_description, proxy, resolveparamsets)
+
+        self.SENSORNODE.update({"CARRIER_SENSE_LEVEL": [0],
+                                "DUTY_CYCLE_LEVEL": [0]})
+        self.BINARYNODE.update({"DUTY_CYCLE": [0]})
+
+    def get_duty_cycle_level(self, channel=None):
+        return float(self.getSensorData("DUTY_CYCLE_LEVEL", channel))
+
+    def get_carrier_sense_level(self, channel=None):
+        return float(self.getSensorData("CARRIER_SENSE_LEVEL", channel))
+
+class TempModuleSTE2(SensorHmIP):
+    """HmIP-STE2-PCB."""
+
+    def __init__(self, device_description, proxy, resolveparamsets=False):
+        super().__init__(device_description, proxy, resolveparamsets)
+
+        # init metadata
+        self.SENSORNODE.update({"ACTUAL_TEMPERATURE ": self.ELEMENT,
+                                "ACTUAL_TEMPERATURE_STATUS ": self.ELEMENT})
+
+    @property
+    def ELEMENT(self):
+        return [1, 2, 3]
 
 DEVICETYPES = {
     "HM-Sec-SC": ShutterContact,
@@ -935,6 +1091,7 @@ DEVICETYPES = {
     "HmIP-SCI": IPShutterContactSabotage,
     "HMIP-SWDO": IPShutterContactSabotage,
     "HmIP-SWDO": IPShutterContactSabotage,
+    "HmIP-SWDO-PL": IPShutterContactSabotage,
     "HmIP-SWDO-I": IPShutterContactSabotage,
     "HmIP-SWDM": IPShutterContact,
     "HmIP-SWDM-B2": IPShutterContact,
@@ -964,10 +1121,11 @@ DEVICETYPES = {
     "263 162": MotionV2,
     "HM-Sec-MD": MotionV2,
     "HmIP-SMI": MotionIP,
-    "HmIP-SMI55": MotionIPV2,
+    "HmIP-SMI55": IPRemoteMotionV2,
     "HmIP-SMO": MotionIP,
     "HmIP-SMO-A": MotionIP,
     "HmIP-SPI": PresenceIP,
+    "HmIPW-SPI": PresenceIPW,
     "HM-Sen-LI-O": LuxSensor,
     "HM-Sen-EP": ImpulseSensor,
     "HM-Sen-X": ImpulseSensor,
@@ -1019,5 +1177,16 @@ DEVICETYPES = {
     "HB-UNI-Sensor1": UniversalSensor,
     "HmIP-FCI1": IPContact,
     "HmIP-FCI6": IPContact,
+    "HmIP-DSD-PCB": IPContact,
     "HB-UNI-Sen-TEMP-DS18B20": TemperatureSensor,
+    "HB-UNI-Sen-WEA": HBUNISenWEA,
+    "HmIP-ASIR-B1": IPAlarmSensor,
+    "HmIP-ASIR-O": IPAlarmSensor,
+    "HmIP-ASIR": IPAlarmSensor,
+    "HmIP-ASIR-2": IPAlarmSensor,
+    "HmIP-FALMOT-C12": ValveBox,
+    "HmIP-SRD": IPRainSensor,
+    "HmIP-HAP": IPLanRouter,
+    "HB-WDS40-THP-O": WeatherStation,
+    "HmIP-STE2-PCB": TempModuleSTE2
 }
